package cz.muni.consumption.database

import android.graphics.Bitmap
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.*
import com.google.firebase.storage.FirebaseStorage
import com.google.firebase.storage.StorageReference
import cz.muni.consumption.data.*
import java.io.ByteArrayOutputStream


object FirebaseProxy {

    private val databaseRef: DatabaseReference = FirebaseDatabase.getInstance().reference
    private val databaseGamesRef: DatabaseReference = databaseRef.child("Games")
    private val databasePlaysRef: DatabaseReference = databaseRef.child("Plays")
    private val databaseInitPlaysRef: DatabaseReference = databaseRef.child("InitPlays")
    private val databasePlayersRef: DatabaseReference = databaseRef.child("Players")

    private val auth: FirebaseAuth = FirebaseAuth.getInstance()

    fun validUsername(username: String, callback: (Boolean) -> Unit) {
        databasePlayersRef.addListenerForSingleValueEvent(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                for (userSnapshot in snapshot.children) {
                    val value = userSnapshot.child("username").value
                    if (value == username) {
                        println("equals")
                        callback(true)
                        return
                    }
                }
                callback(false)
            }

            override fun onCancelled(error: DatabaseError) {
                callback(false)
            }
        })
    }

    fun put(user: User) {
        databasePlayersRef.child(auth.currentUser?.uid.toString())
            .child("username").setValue(user.username)
    }

    fun put(play: Play): String {
        val playsRef = databasePlaysRef.push()
        play.id = playsRef.key!!
        playsRef.setValue(play)
        return play.id
    }

    fun putInitPlay(play: Play): String {
        val playsRef = databaseInitPlaysRef.push()
        play.id = playsRef.key!!
        playsRef.setValue(play)
        return play.id
    }

    fun put(game: Game) {
        val gameRef: DatabaseReference
        if (game.id == "0") {
            gameRef = databaseGamesRef.push()
        } else {
            gameRef = databaseGamesRef.child(game.id)
        }
        game.id = gameRef.key!!
        gameRef.setValue(game)
    }

    fun updateRounds(playId: String, newRounds: List<Round>) {
        FirebaseDatabase.getInstance()
            .getReference("Plays/$playId/rounds")
            .setValue(newRounds)
    }

    fun setPlayFinished(playId: String, finished: Boolean) {
        FirebaseDatabase.getInstance()
            .getReference("Plays/$playId/finished")
            .setValue(finished)
    }

    fun getPlays(success: (List<Play>) -> Unit, fail: () -> Unit) {
        databasePlaysRef.addValueEventListener(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                val plays = mutableListOf<Play>()
                for (playSnapshot in snapshot.children) {
                    val play = playSnapshot.getValue(Play::class.java)
                    if (play != null && play.players.stream()
                            .anyMatch { player -> player.id == auth.currentUser?.uid.toString() }
                    ) {
                        plays.add(play)
                    }
                }
                success(plays)
            }

            override fun onCancelled(error: DatabaseError) {
                fail()
            }
        })
    }

    fun getGames(success: (List<Game>) -> Unit, fail: () -> Unit) {
        databaseGamesRef.addValueEventListener(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                val games = mutableListOf<Game>()
                for (playSnapshot in snapshot.children) {
                    val game = playSnapshot.getValue(Game::class.java)
                    if (game != null) {
                        games.add(game)
                    }
                }
                success(games)
            }

            override fun onCancelled(error: DatabaseError) {
                // Escalate Error
                fail()
            }
        })
    }

    fun updatePlayRoundScores(playId: String, round: Round, roundNo: Int) {
        FirebaseDatabase.getInstance()
            .getReference("Plays/$playId/rounds/$roundNo")
            .setValue(round)
    }

    fun getPlayer(id: String, success: (Player) -> Unit, fail: () -> Unit) {
        val databasePlayerRef: DatabaseReference = databasePlayersRef.child(id)
        databasePlayerRef.addListenerForSingleValueEvent(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                val player = snapshot.getValue(Player::class.java)
                if (player != null) {
                    player.id = id
                    success(player)
                }
            }

            override fun onCancelled(error: DatabaseError) {
                // Escalate Error
                fail()
            }
        })
    }

    fun initPlayPlayerChanged(
        playId: String,
        onAdded: (Player) -> Unit,
        fail: () -> Unit
    ) {
        val playRef = databaseInitPlaysRef.child(playId)
        val playPlayersRef = playRef.child("players")

        playPlayersRef.addChildEventListener(object : ChildEventListener {
            override fun onChildAdded(snapshot: DataSnapshot, previousChildName: String?) {
                val player = snapshot.getValue(Player::class.java)
                if (player != null) {
                    onAdded(player)
                }
            }

            override fun onChildChanged(snapshot: DataSnapshot, previousChildName: String?) {
            }

            override fun onChildRemoved(snapshot: DataSnapshot) {
            }

            override fun onChildMoved(snapshot: DataSnapshot, previousChildName: String?) {
            }

            override fun onCancelled(error: DatabaseError) {
                fail()
            }
        })
    }

    fun putPlayerToInitPlay(playId: String, player: Player) {
        val playRef: DatabaseReference = databaseInitPlaysRef.child(playId)
        val playPlayersRef = playRef.child("players")

        val playerRef = playPlayersRef.push()
        playerRef.setValue(player)
    }

    fun existsInitPlay(playId: String, success: (InitPlay) -> Unit, fail: () -> Unit) {
        if (!isValidFirebasePath(playId)) {
            fail()
            return
        }
        val playRef: DatabaseReference = databaseInitPlaysRef.child(playId)
        playRef.addListenerForSingleValueEvent(object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                val play = dataSnapshot.getValue(InitPlay::class.java)
                if (play != null)
                    success(play)
                else
                    fail()
            }

            override fun onCancelled(error: DatabaseError) {
                fail()
            }
        })
    }

    private fun isValidFirebasePath(path: String): Boolean {
        return path.startsWith("-") &&
                !(path.contains(".")
                        || path.contains("#")
                        || path.contains("$")
                        || path.contains("[")
                        || path.contains("]"))
    }

    fun deleteInitPlay(playId: String) {
        val playRef: DatabaseReference = databaseInitPlaysRef.child(playId)
        playRef.removeValue()
    }

    fun deletePlay(playId: String) {
        val playRef: DatabaseReference = databasePlaysRef.child(playId)
        playRef.removeValue()
    }

    fun deleteGame(gameId: String) {
        val gameRef: DatabaseReference = databaseGamesRef.child(gameId)
        gameRef.removeValue()
    }

    fun finishInitializingPlay(initPlayId: String, selectedPlayers: List<Player>, playId: String) {
        val playRef: DatabaseReference = databaseInitPlaysRef.child(initPlayId)

        var id = 1
        val playerMap = mutableMapOf<String, Player>()
        for (player in selectedPlayers) {
            var currentId: String
            if (player.id.isNotEmpty()) {
                currentId = player.id
            } else {
                currentId = id.toString()
                id++
            }
            playerMap[currentId] = player
        }

        val childUpdates: MutableMap<String, Any> = HashMap()
        childUpdates["/players"] = playerMap
        childUpdates["/playId"] = playId

        playRef.updateChildren(childUpdates)
    }

    fun getPlay(id: String, onSuccess: (Play) -> Unit, onFail: () -> Unit) {
        val databasePlayRef: DatabaseReference = databasePlaysRef.child(id)
        databasePlayRef.addListenerForSingleValueEvent(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                val play = snapshot.getValue(Play::class.java)
                if (play != null) {
                    onSuccess(play)
                }
            }

            override fun onCancelled(error: DatabaseError) {
                onFail()
            }
        })
    }

    fun getPlayList(playId: String, success: (List<Play>) -> Unit, fail: () -> Unit) {
        databasePlaysRef.addValueEventListener(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                val plays = mutableListOf<Play>()
                val playSnapshot = snapshot.child(playId)
                val play = playSnapshot.getValue(Play::class.java)
                if (play != null) {
                    plays.add(play)
                }
                success(plays)
            }

            override fun onCancelled(error: DatabaseError) {
                fail()
            }
        })
    }

    fun getImageRef(gameId: String): StorageReference {
        return FirebaseStorage.getInstance().reference.child("${gameId}.jpg")
    }

    fun put(picture: Bitmap, name: String, success: () -> Unit, fail: () -> Unit) {
        val storageRef = FirebaseStorage.getInstance().reference.child("$name.jpg")

        val outputStream = ByteArrayOutputStream()
        picture.compress(Bitmap.CompressFormat.JPEG, 100, outputStream)
        val data = outputStream.toByteArray()

        val uploadTask = storageRef.putBytes(data)
        uploadTask.addOnCompleteListener { task ->
            if (task.isSuccessful) {
                success()
            } else {
                fail()
            }
        }
    }

}