package cz.muni.consumption.ui.games.detail

import android.app.Activity.RESULT_OK
import android.content.ActivityNotFoundException
import android.content.Intent
import android.graphics.Bitmap
import android.os.Bundle
import android.provider.MediaStore
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import androidx.navigation.fragment.navArgs
import cz.muni.consumption.data.Game
import cz.muni.consumption.database.FirebaseProxy
import cz.muni.consumption.databinding.FragmentAddEditGameDetailBinding


class GameDetailAddEditFragment : Fragment() {

    private lateinit var binding: FragmentAddEditGameDetailBinding
    private val args: GameDetailAddEditFragmentArgs by navArgs()
    private lateinit var game: Game
    private lateinit var takePictureLauncher: ActivityResultLauncher<Intent>
    private lateinit var picture: Bitmap

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentAddEditGameDetailBinding.inflate(layoutInflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        setInitialValues()

        val actionBar = (activity as AppCompatActivity).supportActionBar
        actionBar?.title = "Game"

        takePictureLauncher = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == RESULT_OK) {
                picture = result.data?.extras?.getParcelable("data")!!
                binding.imageview.setImageBitmap(picture)
            } else {
                Toast.makeText(requireContext(), "Picture Launcher failed.", Toast.LENGTH_SHORT).show()
            }
        }

        binding.photoButton.setOnClickListener {
            dispatchTakePictureIntent()
        }

        binding.btnSave.setOnClickListener {
            val gameName = binding.gameName.text.toString()
            val minPlayersText = binding.noOfPlayersFrom.text.toString()
            val maxPlayersText = binding.noOfPlayersTo.text.toString()
            val minTimeText = binding.timeDurationFrom.text.toString()
            val maxTimeText = binding.timeDurationTo.text.toString()
            val cardType = binding.cardTypeValue.text.toString()
            val description = binding.description.text.toString()
            val rules = binding.rules.text.toString()
            val substractText = binding.scoringSpinner.selectedItem.toString()
            val scoreValueText = binding.winNumber.text.toString()

            if (isGameValid(gameName, minPlayersText, maxPlayersText, minTimeText, maxTimeText, cardType, description, rules, scoreValueText)) {
                val minPlayers = minPlayersText.toInt()
                val maxPlayers = maxPlayersText.toInt()
                val minTime = minTimeText.toInt()
                val maxTime = maxTimeText.toInt()
                var scoreValue = 0
                val scoreSubtract = substractText == "Decreasing"
                if (scoreValueText != "")
                {
                    if (scoreSubtract) {
                        scoreValue = scoreValueText.toInt()
                    }
                }

                game.name = gameName
                game.cardType = cardType
                game.description = description
                game.rules = rules
                game.maxPlayers = maxPlayers
                game.minPlayers = minPlayers
                game.maxTime = maxTime
                game.minTime = minTime
                game.scoreSubtract = scoreSubtract
                game.scoreValue = scoreValue

                FirebaseProxy.put(game) // sets the id
                if (::picture.isInitialized) {
                    FirebaseProxy.put(picture, game.id,
                        success = {
                            // update picture in game detail
                        },
                        fail = {
                            Toast.makeText(
                                requireContext(),
                                "Picture could not be saved",
                                Toast.LENGTH_SHORT
                            ).show()
                        })
                }

                Toast.makeText(requireContext(), "${game.name} was successfully saved", Toast.LENGTH_SHORT).show()
                findNavController().navigateUp()
            }
        }
    }

    private fun dispatchTakePictureIntent() {
        val takePictureIntent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
        try {
            takePictureLauncher.launch(takePictureIntent)
        } catch (e: ActivityNotFoundException) {
            Toast.makeText(requireContext(), "Picture Launcher Activity not found", Toast.LENGTH_SHORT).show()
        }
    }

    private fun setInitialValues() {
        if (args.game == null) {
            game = Game()
        } else {
            // the changes will propagate to GameDetailFragment
            game = args.game!!

            binding.gameName.setText(game.name)
            binding.noOfPlayersFrom.setText(game.minPlayers.toString())
            binding.noOfPlayersTo.setText(game.maxPlayers.toString())
            binding.timeDurationFrom.setText(game.minTime.toString())
            binding.timeDurationTo.setText(game.maxTime.toString())
            binding.cardTypeValue.setText(game.cardType)
            binding.description.setText(game.description)
            binding.rules.setText(game.rules)
            binding.winNumber.setText(game.scoreValue.toString())

            val index = if (game.scoreSubtract) 1 else 0
            binding.scoringSpinner.setSelection(index)
        }
    }

    private fun isGameValid(gameName: String, minPlayers: String, maxPlayers: String,
                            minTime: String, maxTime: String, cardType: String,
                            description: String, rules: String, scoreValue: String): Boolean {
        if (gameName.isBlank()) {
            binding.gameName.error = "Field is not valid"
            return false
        }

        if (minPlayers.isEmpty() || minPlayers.toIntOrNull() === null) {
            binding.noOfPlayersFrom.error = "Field is not valid"
            return false
        }

        if (maxPlayers.isEmpty() || maxPlayers.toIntOrNull() === null) {
            binding.noOfPlayersTo.error = "Field is not valid"
            return false
        }

        if (minTime.isEmpty() || minTime.toIntOrNull() === null) {
            binding.timeDurationFrom.error = "Field is not valid"
            return false
        }

        if (maxTime.isEmpty() || maxTime.toIntOrNull() === null) {
            binding.timeDurationTo.error = "Field is not valid"
            return false
        }

        if (cardType.isBlank()) {
            binding.cardTypeValue.error = "Field is not valid"
            return false
        }

        if (description.isBlank()) {
            binding.description.error = "Field is not valid"
            return false
        }

        if (rules.isBlank()) {
            binding.rules.error = "Field is not valid"
            return false
        }

        if (scoreValue.isNotEmpty()) {
            if (scoreValue.toIntOrNull() === null) {
                binding.winNumber.error = "Field is not valid"
                return false
            }
        }

        return true
    }

}