package cz.muni.consumption.ui.playSession.selectPlayers

import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context.CLIPBOARD_SERVICE
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import androidx.navigation.fragment.navArgs
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.firebase.auth.FirebaseAuth
import cz.muni.consumption.RoundActivity
import cz.muni.consumption.data.Play
import cz.muni.consumption.data.Player
import cz.muni.consumption.database.FirebaseProxy
import cz.muni.consumption.databinding.FragmentSelectPlayersBinding


class SelectPlayersFragment : Fragment() {
    private lateinit var binding: FragmentSelectPlayersBinding

    private val args: SelectPlayersFragmentArgs by navArgs()

    private val selectedPlayers = mutableListOf<Player>()
    private val playerList = mutableListOf<Player>()
    private lateinit var play : Play
    private lateinit var initPlayId : String

    private val adapter: PlayerAdapter by lazy {
        PlayerAdapter(
            playerList = playerList,
            onPlayerChecked = { player, isChecked ->
                if (isChecked) {
                    selectedPlayers.add(player)
                    Toast.makeText(requireContext(), "${player.username} was added to the game", Toast.LENGTH_SHORT).show()
                } else {
                    selectedPlayers.remove(player)
                    Toast.makeText(requireContext(), "${player.username} was removed from the game", Toast.LENGTH_SHORT).show()
                }
            },
        )
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentSelectPlayersBinding.inflate(layoutInflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setInitialValues()
    }

    private fun setInitialValues() {
        binding.playerList.layoutManager = LinearLayoutManager(context)
        binding.playerList.adapter = adapter

        val game = args.game
        play =  Play(game.id, game.name, game.scoreSubtract, game.scoreValue)
        initPlayId = FirebaseProxy.putInitPlay(play)
        binding.code.setText(initPlayId)
        binding.code.isEnabled = false

        binding.btnAdd.setOnClickListener {
            val guestName = binding.editGuestName.text.toString()
            binding.editGuestName.setText("")
            if (guestName != "") {
                if (!isUniqueName(guestName)) {
                    Toast.makeText(
                        requireContext(),
                        "Name already exists in the play",
                        Toast.LENGTH_SHORT
                    ).show()
                } else {
                    val newPlayer = Player(guestName)
                    FirebaseProxy.putPlayerToInitPlay(initPlayId, newPlayer)
                    Toast.makeText(
                        requireContext(),
                        "${newPlayer.username} was added to the game",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }
        }

        binding.btnNext.setOnClickListener {
            createPlay()
            /*findNavController()
                .navigate(SelectPlayersFragmentDirections.actionSelectPlayersFragmentToRoundFragment(play = play))*/
            val intent = Intent(activity, RoundActivity::class.java)
            //val bundle = Bundle()
            //bundle.putString("playId", play.id)
            //intent.putExtra("playId", play.id)
            intent.putExtra("play", play)
            startActivity(intent)
            findNavController().popBackStack(
                findNavController().graph.startDestinationId, false
            )
            //(activity as Activity?)!!.overridePendingTransition(0, 0)

        }

        binding.btnCopy.setOnClickListener {
            val clipboard =
                requireContext().getSystemService(CLIPBOARD_SERVICE) as ClipboardManager
            val clip = ClipData.newPlainText("Copied Token", binding.code.text.toString())
            clipboard.setPrimaryClip(clip)
            Toast.makeText(context, "Token copied", Toast.LENGTH_SHORT).show()
        }

        val currentUser = FirebaseAuth.getInstance().currentUser
        val id = currentUser?.uid
        if (id == null) {
            Toast.makeText(context, "Server error fetching current user", Toast.LENGTH_SHORT).show()
        } else {
            selectPlayer(id)
        }
    }

    private fun createPlay() {
        selectedPlayers.forEach { player ->
            play.addPlayer(player)
        }
        val newPlayId = FirebaseProxy.put(play)
        FirebaseProxy.finishInitializingPlay(initPlayId, selectedPlayers, newPlayId)
        FirebaseProxy.deleteInitPlay(initPlayId)
    }

    private fun isUniqueName(questName: String): Boolean {
        selectedPlayers.forEach { player ->
            if (player.username == questName) {
                return false
            }
        }
        return true
    }

    private fun selectPlayer(id: String) {
        FirebaseProxy.getPlayer(
            id,
            success = { player ->
                FirebaseProxy.putPlayerToInitPlay(initPlayId, player)
            },
            fail = {
                Toast.makeText(context, "Server error", Toast.LENGTH_SHORT).show()
            }
        )
    }

    private fun refreshList() {
        FirebaseProxy.initPlayPlayerChanged(
            playId = initPlayId,
            onAdded = { newPlayer ->
                if (!playerList.contains(newPlayer) || !selectedPlayers.contains(newPlayer)) {
                    playerList.add(newPlayer)
                    selectedPlayers.add(newPlayer)
                    val index = playerList.size - 1
                    adapter.notifyItemInserted(index)
                }
            },
            fail = {
                Toast.makeText(context, "Server error", Toast.LENGTH_SHORT).show()
            }
        )
    }

    override fun onResume() {
        super.onResume()
        refreshList()
    }

}